Anyway, thanks a lot for this experiment. I rarely use MSVC these days. It is good to know where it stands.
OK, I’ve installed Cygwin and GCC, compiled and benchmarked the original code. I made the following changes in the makefile: (1) Replaced -O2 with -O3 (2) added -msse -msse2 -msse3 -mssse3 -msse4 -msse4.1 to the option, both C and C++.
The results on GCC/i5/Windows 10 are very consistent with the OPs result on GCC/Xeon/Linux.
Diagram: https://raw.githubusercontent.com/Const-me/matmul/master/res...
Numbers: https://github.com/Const-me/matmul/blob/master/Run/result.xl...
while that could be the case, it does not follow from your results; you need to compare GCC and MSVC directly. It might rather be that MSVC is not capable to optimize Eigen enough to give it a larger advantage over SSE+tiling.
From experience MSVC is less capable of aggressive abstraction elimination (the bread and butter of Eigen) than GCC.
GCC indeed did better optimization of Eigen than MSVC. Specifically, about 25% better. I think that might be because authors of Eigen have very intimate knowledge on GCC’s optimizer, and somehow specifically optimized their code for GCC compiler.
However, for the rest of the test cases, MSVC performed much better. For example, for Transposed algorithm, MSVC result is 3.5 times faster - huge difference.
Here’s Visual C++ port: https://github.com/Const-me/matmul/
Eugen is still faster than naïve implementations, but not that faster, just 30-40% compared to SSE+tiling sdot.